-----------------------------------------------------------------
--  video.vhd
--			Video-Interface fuer LCD124
--			===========================
--
--  07.10.97	Bodo Wenzel	Erstellung
--  24.03.98	Bodo Wenzel	Hardware fertig
--  06.05.98	Bodo Wenzel	Kein "restart" mehr
--  26.05.98	Bodo Wenzel	Jumper fuer Frequenz und Bildlage
--  29.06.98	Bodo Wenzel	Konstanten justiert
-----------------------------------------------------------------

-- Interface ----------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity video is
  port (clock_p:  in  std_ulogic;
        clock_b:  in  std_ulogic;
        vsync_jp: in  std_ulogic_vector(4 downto 0);
        hfreq_jp: in  std_ulogic_vector(1 downto 0);
        hsync_jp: in  std_ulogic_vector(1 downto 0);
        pixel_jp: in  std_ulogic_vector(2 downto 0);
        vsync_x:  in  std_ulogic;
        hsync_x:  in  std_ulogic;
        hsync_i:  out std_ulogic;
        mono:     in  std_ulogic;
        ctl_wr:   in  std_ulogic;
        addr:     out std_ulogic_vector(14 downto 0);
        data:     out std_ulogic_vector(7 downto 0);
        n_we:     out std_ulogic;
        catching: out std_ulogic);
end;

-- Implementation -----------------------------------------------

architecture beh of video is
  constant H_SYNC:    natural :=              10;
  constant H_DSP_BEG: natural :=              29;
  constant H_DSP_END: natural := H_DSP_BEG +  80;
  constant H_RESTART: natural :=             113;
  constant H_MAX:     natural :=             127;

  constant V_MAX:     natural := 63;

  constant ADDR_MAX:  unsigned(14 downto 0)
                   := conv_unsigned(31999,15);

  signal v_incr: std_ulogic;

  signal h_count: natural range 0 to H_MAX := 0;
  signal v_count: natural range 0 to V_MAX := 0;

  signal de_h0: std_ulogic;
  signal de_h1: std_ulogic;
  signal de_h2: std_ulogic;
  signal de_h3: std_ulogic;
  signal de_h:  std_ulogic;
  signal de_v1: std_ulogic;
  signal de_v2: std_ulogic;

  signal i_addr: unsigned(14 downto 0) := (others => '0');

  signal p_pixel: std_ulogic_vector(7 downto 0);
  signal p_shift: std_ulogic_vector(7 downto 0);
begin
  synchronize: process (clock_b)
  begin
    if rising_edge(clock_b) then
      h_count <= h_count+1;

      case h_count is
      when H_SYNC =>
        hsync_i <= '0';
      when H_RESTART-1 =>
        hsync_i <= '1';
        h_count <= conv_integer(unsigned(hfreq_jp));
      when others =>
        null;
      end case;
    end if;
  end process;

  horizontal: process (clock_b)
  begin
    if rising_edge(clock_b) then
      case hsync_jp is
      when "00" =>
        de_h <= de_h0;
      when "01" =>
        de_h <= de_h1;
      when "10" =>
        de_h <= de_h2;
      when others =>
        de_h <= de_h3;
      end case;

      de_h3 <= de_h2;
      de_h2 <= de_h1;
      de_h1 <= de_h0;

      v_incr <= '0';

      case h_count is
      when H_DSP_BEG =>
        catching <= not hsync_x;
        de_h0    <= de_v1 and de_v2;
      when H_DSP_END =>
        de_h0    <= '0';
        v_incr   <= '1';
      when others =>
        null;
      end case;
    end if;
  end process;

  vertical: process (vsync_x,clock_b)
  begin
    if vsync_x='1' then
      de_v1   <= '0';
      de_v2   <= '1';
      v_count <= 0;
      i_addr  <= (others => '0');
    elsif rising_edge(clock_b) then
      if v_incr='1' then
        if v_count=unsigned('1' & vsync_jp) then
          de_v1 <= '1';
        end if;

        v_count <= v_count+1;
      end if;

      if de_h='1' then
        if i_addr=ADDR_MAX then
          de_v2 <= '0';
        end if;

        i_addr <= i_addr+1;
      end if;
    end if;
  end process;

  addr <= std_ulogic_vector(i_addr);

  shifter: process (clock_p)
  begin
    if falling_edge(clock_p) then
      p_shift <= p_shift(6 downto 0) &
                 p_pixel(conv_integer(unsigned(pixel_jp)));
      p_pixel <= p_pixel(6 downto 0) & (not mono);
    end if;
  end process;

  latch: process (clock_b)
  begin
    if falling_edge(clock_b) then
      data <= p_shift;
    end if;
  end process;

  n_we <= not (de_h and ctl_wr);
end;

-- Ende ---------------------------------------------------------
